#include <Servo.h>

//
// Plank Robot Code
// Eric Fialkowski
// 
// Version  Date      Notes
// .1       10/4/08   Initial work
// .2       10/5/08   Trying to correct for drift + randomize the turn direction on object
// .3       10/5/08   Smoothing out forward movement + randomize backup
// .4       10/5/08   Added bumpers
// .5       10/5/08   Added some movement randomness
// .6       10/6/08   Implemented bumper switch interrupts + potentiometer for move randomness

int potPin = 3;
int rightIrSensor = 2;
int centerIrSensor = 1;
int leftIrSensor = 0;
int ledPin = 13;   
int rightServoPin = 10;
int leftServoPin = 9;
int rightBumperPin = 2;
int leftBumperPin = 3;
Servo rightServo;
Servo leftServo;

//
// Called on a bumper switch interrupt
//  - stops the motion
//
void bumperInterrupt()
{
  stop();
}

//
// Forumla from Acroname robotics
// http://www.acroname.com/robotics/info/articles/irlinear/irlinear.html
//
int convert_gp2d120_range(int raw) {
  return (2914 / (raw + 5)) - 1;
} 

//
// Helper
//
boolean frontSensorHit(byte safeRange)
{
  static long lastRead = 0;
  static boolean lastVal = true;

  // make sure we don't read it within 10 ms of the last time
  if ( millis() > lastRead + 10)
  {
    lastRead = millis();
    int val = convert_gp2d120_range(analogRead(centerIrSensor));
    lastVal = ( val < safeRange );
  }
  return lastVal;
}

boolean rightSensorHit(byte safeRange)
{
  static long lastRead = 0;
  static boolean lastVal = true;

  // make sure we don't read it within 10 ms of the last time
  if ( millis() > lastRead + 10)
  {
    lastRead = millis();
    int val = convert_gp2d120_range(analogRead(rightIrSensor));
    lastVal = ( val < safeRange );
  }
  return lastVal;
}

boolean leftSensorHit(byte safeRange)
{
  static long lastRead = 0;
  static boolean lastVal = true;

  // make sure we don't read it within 10 ms of the last time
  if ( millis() > lastRead + 10)
  {
    lastRead = millis();
    int val = convert_gp2d120_range(analogRead(leftIrSensor));
    lastVal = ( val < safeRange );
  }
  return lastVal;
}


//
// Helper to flash the LED
//
void flashLed(byte times)
{
  for (int i = 0; i < times; i++)
  {  
    digitalWrite(ledPin, HIGH);   // sets the LED on
    delay(150);                  // waits for a second
    digitalWrite(ledPin, LOW);    // sets the LED off
    delay(150);                  // waits for a second
  }  
}

//
// Centers the servos (stopping them)
//
void stop()
{
  rightServo.write(90);
  leftServo.write(90);    
}

void moveForward(int howFar)
{
  rightServo.write(35);
  leftServo.write(135);
  delay(howFar * 10);
  //stop();
}

void moveBackwards(int howFar)
{
  rightServo.write(135);
  leftServo.write(45);
  delay(howFar * 10);
  //stop();
}

void turnRight(int howFar)
{
  rightServo.write(45);
  leftServo.write(45);
  delay(howFar * 10);
}

void pivotRight(int howFar)
{
  rightServo.write(90);
  leftServo.write(45);
  delay(howFar * 10);
}

void turnLeft(int howFar)
{
  rightServo.write(135);
  leftServo.write(135);
  delay(howFar * 10);
}

void pivotLeft(int howFar)
{
  rightServo.write(135);
  leftServo.write(90);
  delay(howFar * 10);
}



//
// setup code
//
void setup()
{
  Serial.begin(9600);
  randomSeed(millis());
  
  // pin modes
  pinMode(ledPin, OUTPUT);
  pinMode(rightBumperPin, OUTPUT);
  pinMode(leftBumperPin, OUTPUT);
  
  // turn on the internal pull up
  digitalWrite(rightBumperPin,HIGH);
  digitalWrite(leftBumperPin,HIGH);
  
  // servo set up
  rightServo.attach(rightServoPin);
  leftServo.attach(leftServoPin);
  stop();
  
  // interrupts
  attachInterrupt(0, bumperInterrupt, FALLING);
  attachInterrupt(1, bumperInterrupt, FALLING);
  
  // in case of error, we'll see the flashing over and over
  flashLed(3);  
}

void loop()
{
  //
  // Check sensors
  //
  if ( digitalRead(rightBumperPin) == LOW)
  {
    stop();
    Serial.println("RB");
    turnRight(30);
    moveBackwards(random(50,76));
  }
  if ( digitalRead(leftBumperPin) == LOW)
  {
    stop();
    Serial.println("LB");
    turnLeft(30);
    moveBackwards(random(50,76));
  }
  if ( leftSensorHit(10) )
  {
    stop();
    pivotRight(random(10,26));
  }
  if ( rightSensorHit(10) )
  {
    stop();
    pivotLeft(random(10,26));
  }
  if ( frontSensorHit(25) )
  {
    stop();
    moveBackwards(random(50,76));
    if ( random(1,3) == 1)
    {
      turnRight(60);
    }
    else
    {
      turnLeft(60);
    }
  }
  
  //
  // This is for fun randomness to the movement
  //
  int pot = analogRead(potPin);
  if ( pot < 7 )
  {
    pot = 7;
  }
  int dir = random(1,pot);
  switch (dir)
  {
    case 1:
      moveBackwards(random(20,31));
      break;
    case 2:
      turnRight(random(10,26));
      break;
    case 3:
      turnLeft(random(10,26));
      break;
    case 4:
      pivotRight(random(10,26));
      break;
    case 5:
      pivotLeft(random(10,26));
      break;      
    default:
      moveForward(random(20,36));
      break;
  }
}
